@@ -1,5 +1,5 @@
 use crate::build_options::{extract_cargo_metadata_args, CargoOptions};
-use crate::{CargoToml, Metadata21, PyProjectToml};
+use crate::{CargoToml, Metadata23, PyProjectToml};
 use anyhow::{bail, format_err, Context, Result};
 use cargo_metadata::{Metadata, MetadataCommand};
 use normpath::PathExt as _;
@@ -7,13 +7,15 @@ use std::collections::HashSet;
 use std::env;
 use std::io;
 use std::path::{Path, PathBuf};
-use tracing::debug;
+use tracing::{debug, instrument};
 
 const PYPROJECT_TOML: &str = "pyproject.toml";
 
 /// Whether this project is pure rust or rust mixed with python and whether it has wheel data
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct ProjectLayout {
+    /// Contains the absolute path to the project root directory
+    pub project_root: PathBuf,
     /// Contains the absolute path to the python source directory
     pub python_dir: PathBuf,
     /// Contains the canonicalized (i.e. absolute) path to the python part of the project
@@ -45,8 +47,8 @@ pub struct ProjectResolver {
     pub pyproject_toml: Option<PyProjectToml>,
     /// Rust module name
     pub module_name: String,
-    /// Python Package Metadata 2.1
-    pub metadata21: Metadata21,
+    /// Python Package Metadata 2.3
+    pub metadata23: Metadata23,
     /// Cargo options
     pub cargo_options: CargoOptions,
     /// Cargo.toml as resolved by [cargo_metadata]
@@ -90,7 +92,7 @@ impl ProjectResolver {
         let manifest_dir = manifest_file.parent().unwrap();
         let pyproject_toml: Option<PyProjectToml> = if pyproject_file.is_file() {
             let pyproject = PyProjectToml::new(&pyproject_file)?;
-            pyproject.warn_missing_maturin_version();
+            pyproject.warn_bad_maturin_version();
             pyproject.warn_missing_build_backend();
             Some(pyproject)
         } else {
@@ -107,11 +109,11 @@ impl ProjectResolver {
 
         let cargo_metadata = Self::resolve_cargo_metadata(&manifest_file, &cargo_options)?;
 
-        let mut metadata21 = Metadata21::from_cargo_toml(manifest_dir, &cargo_metadata)
+        let mut metadata23 = Metadata23::from_cargo_toml(manifest_dir, &cargo_metadata)
             .context("Failed to parse Cargo.toml into python metadata")?;
         if let Some(pyproject) = pyproject {
             let pyproject_dir = pyproject_file.parent().unwrap();
-            metadata21.merge_pyproject_toml(pyproject_dir, pyproject)?;
+            metadata23.merge_pyproject_toml(pyproject_dir, pyproject)?;
         }
 
         let crate_name = &cargo_toml.package.name;
@@ -193,7 +195,7 @@ impl ProjectResolver {
             pyproject_toml_path: pyproject_file,
             pyproject_toml,
             module_name,
-            metadata21,
+            metadata23,
             cargo_options,
             cargo_metadata,
             pyproject_toml_maturin_options,
@@ -209,7 +211,7 @@ impl ProjectResolver {
         if let Some(path) = cargo_manifest_path {
             let path = path
                 .normalize()
-                .with_context(|| format!("failed to normalize path `{}`", path.display()))?
+                .with_context(|| format!("failed to normalize manifest path `{}`", path.display()))?
                 .into_path_buf();
             debug!(
                 "Using cargo manifest path from command line argument: {:?}",
@@ -248,7 +250,9 @@ impl ProjectResolver {
                 debug!("Using cargo manifest path from pyproject.toml {:?}", path);
                 return Ok((
                     path.normalize()
-                        .with_context(|| format!("failed to normalize path `{}`", path.display()))?
+                        .with_context(|| {
+                            format!("failed to normalize manifest path `{}`", path.display())
+                        })?
                         .into_path_buf(),
                     pyproject_file,
                 ));
@@ -310,6 +314,7 @@ impl ProjectResolver {
         }
     }
 
+    #[instrument(skip_all)]
     fn resolve_cargo_metadata(
         manifest_path: &Path,
         cargo_options: &CargoOptions,
@@ -317,6 +322,10 @@ impl ProjectResolver {
         debug!("Resolving cargo metadata from {:?}", manifest_path);
         let cargo_metadata_extra_args = extract_cargo_metadata_args(cargo_options)?;
         let result = MetadataCommand::new()
+            // Force resolving metadata using cargo instead of instead of $CARGO env var
+            // to avoid getting wrong file path like target directory, for example `cross` would
+            // output paths in docker while we want paths on host.
+            .cargo_path("cargo")
             .manifest_path(manifest_path)
             .verbose(true)
             .other_options(cargo_metadata_extra_args)
@@ -390,6 +399,7 @@ impl ProjectLayout {
             eprintln!("🍹 Building a mixed python/rust project");
 
             Ok(ProjectLayout {
+                project_root: project_root.to_path_buf(),
                 python_dir: python_root,
                 python_packages,
                 python_module: Some(python_module),
@@ -408,6 +418,7 @@ impl ProjectLayout {
             }
 
             Ok(ProjectLayout {
+                project_root: project_root.to_path_buf(),
                 python_dir: python_root,
                 python_packages,
                 python_module: None,
